#!/usr/bin/env python

import matplotlib.pyplot as plt
import matplotlib.tri as tri
import numpy as np
import pandas as pd
import matplotlib.ticker as ticker
import scipy.ndimage
from scipy.ndimage.filters import gaussian_filter
from scipy.interpolate import interp1d, griddata
from matplotlib.gridspec import GridSpec, GridSpecFromSubplotSpec


plt.rcParams["font.family"] = "Arial"    # フォント設定
plt.rcParams["mathtext.cal"] = "Arial"       # TeX表記に関するフォント設定
plt.rcParams["mathtext.rm"] = "Arial"       # TeX表記に関するフォント設定
plt.rcParams["mathtext.it"] = "Arial"      # TeX表記に関するフォント設定
plt.rcParams["mathtext.bf"] = "Arial"    # TeX表記に関するフォント設定


#測点間を計算するための関数を作成しておく

#二点間
def cal_rho(lon_a,lat_a,lon_b,lat_b):
    if lon_a == lon_b and lat_a == lat_b:
        return 0
    ra=6378.140  # equatorial radius (km)
    rb=6356.755  # polar radius (km)
    F=(ra-rb)/ra # flattening of the earth
    rad_lat_a=np.radians(lat_a)
    rad_lon_a=np.radians(lon_a)
    rad_lat_b=np.radians(lat_b)
    rad_lon_b=np.radians(lon_b)
    pa=np.arctan(rb/ra*np.tan(rad_lat_a))
    pb=np.arctan(rb/ra*np.tan(rad_lat_b))
    xx=np.arccos(np.sin(pa)*np.sin(pb)+np.cos(pa)*np.cos(pb)*np.cos(rad_lon_a-rad_lon_b))
    c1=(np.sin(xx)-xx)*(np.sin(pa)+np.sin(pb))**2/np.cos(xx/2)**2
    c2=(np.sin(xx)+xx)*(np.sin(pa)-np.sin(pb))**2/np.sin(xx/2)**2
    dr=F/8*(c1-c2)
    rho=ra*(xx+dr)
    return rho

def dis(st, n, m, name1, name2): #n, mは経度と緯度のカラム番号に対応させる
    #前後の距離を計算する
    distance = []
    for i in range(len(st)):
        if i == 0:
            new = 0
        else:
             new = cal_rho(st.iat[i,n], st.iat[i,m], st.iat[i-1, n], st.iat[i-1, m])
        distance.append(new)
    st[name1] = distance
    
    # それぞれを足し合わせて使う（こうすることで曲がっているトランゼくとにも対応可能）
    kyo = []
    Sum = 0 
    for i in range(len(st)):
        if i ==0:
            Sum = 0
        else:
            Sum = Sum + st[name1][i]
        kyo.append(Sum)
    st[name2] = kyo 
    return st

############### Main program

df = pd.read_excel('CTDdata.xlsx', sheet_name='ctd')

name_line = "B"  ## AA
Station = df[df['line'] == name_line].reset_index(drop=True) #reset=index忘れがちなので注意！
#Station.head()

name_variable = "sal" ## chla, temp, sal

dis(Station, 3, 2, 'peri', 'dis')

#カラーパレットの作成（デフォルトであるが、佐藤は自分のお気に入りのパレットを作って使ってます）
import matplotlib.colors as clr
cdict3 = {'red':((0.0,1.0,1.0),
                 (0.5,0.25,0.25),
                 (1.0,1.0,1.0)),
          'green':((0.0,1.0,1.0),
                   (0.5,0.5,0.5),
                   (0.75,1.0,1.0),
                   (1.0,0.25,0.25)),
          'blue':((0.0,1.0,1.0),
                  (0.5,0.75,0.75),
                  (0.75,0.0,0.0),
                  (1.0,0.0,0.0))}
mycolor1 = clr.LinearSegmentedColormap('mycolor1', cdict3)

fig, ax = plt.subplots(figsize=(12,5))

fs = 12

lat, depth = Station['dis'], Station['depth']
z = Station[name_variable]
lati = np.arange(min(Station['dis']), max(Station['dis']), .5)
depthi = np.arange(0, 1001, 2)
lati, depthi = np.meshgrid(lati, depthi)
zi = griddata((lat,depth),z,(lati,depthi), method= 'cubic')


## contour は、等値線の関数
if name_variable == "chla":
    cs = ax.contour(lati, depthi, zi, colors = 'k', levels = [0.03, 0.05, 0.1, 0.2], linewidths = .7, linestyles = '-')  # chla
elif name_variable == "temp":
    cs = ax.contour(lati, depthi, zi, colors = 'k', levels = [13, 15, 20, 30], linewidths = .7, linestyles = '-')  # temp
else:
    cs = ax.contour(lati, depthi, zi, colors = 'k', levels = [34, 34.3, 34.6, 35], linewidths = .7, linestyles = '-')  # sal

ax.clabel(cs , inline = 1, fmt = '%.2f', fontsize = 10)
if name_variable == "chla":
    c = ax.pcolormesh(lati, depthi, zi, cmap = mycolor1, alpha = 1, vmin = 0, vmax = .2)  # chla
elif name_variable == "temp":
    c = ax.pcolormesh(lati, depthi, zi, cmap = 'rainbow', alpha = 1, vmin = 13, vmax = 30) # Temparature
    #c = ax.pcolormesh(lati, depthi, zi, cmap = mycolor1, alpha = 1, vmin = 13, vmax = 30) # Temparature
else:
    #c = ax.pcolormesh(lati, depthi, zi, cmap = "rainbow", alpha = 1, vmin = 34, vmax = 35) # Sal
    #c = ax.pcolormesh(lati, depthi, zi, cmap = "Blues", alpha = 1, vmin = 34, vmax = 35) # Sal
    c = ax.pcolormesh(lati, depthi, zi, cmap = "rainbow", alpha = 1, vmin = 34, vmax = 35) # Sal


ax.scatter(lat, depth , c ='k' , s = 3, alpha = 0.4)

#ax.set_ylim(200, 0)
ax.set_ylim(1000, 0)

ax.set_xticks(np.arange(0, 100, 50))
ax.set_yticks([0, 200, 400, 600, 800, 1000])

# pad はグラフとグラデーションバーの間を決める
if name_variable == "chla":
    cbar = fig.colorbar(c, ax=ax, extend='both', pad = .02, ticks =[0, 0.05, 0.1, 0.15, 0.2])  # Chla
elif name_variable == "temp":
    cbar = fig.colorbar(c, ax=ax, extend='both', pad = .02, ticks =[10, 15, 20, 25, 30])  # Temp
else:
    cbar = fig.colorbar(c, ax=ax, extend='both', pad = .02, ticks =[34, 34.25, 34.5, 34.75, 35])  # Sal

if name_variable == "chla":
    cbar.set_label(label ='Chl $\it{a}$ [µg $\mathrm{L^{{-}1}}$]', fontsize = fs)  # Chl
elif name_variable == "temp":
    cbar.set_label(label ='Temp [C]', fontsize = fs)   # Temp
else:
    cbar.set_label(label ='Salinity', fontsize = fs)   # Sal

#測点名を上に書く
st = Station.drop_duplicates(subset='station').reset_index(drop =True) #station情報のみのデータファイルを作成
h = -5
#for j in range(len(st)):
#    ax.text(st['dis'][j] - 5, h, st['station'][j], c = 'k', fontsize = fs) 
for j in range(len(st)):
    ax.text(st['dis'][j] - 5, h, st['station'][j], fontsize = fs) 
    

#######海底を作る。水深が深ければ不要。 ###
#ax.fill_between(st['dis'],st['水深'],300, color = 'sienna', edgecolor = 'k')
#ax.plot(st['dis'],st['水深'], color = 'k', lw = 0.7)


#軸名
ax.set_ylabel('Depth [m]', fontsize = fs)
ax.set_xlabel('Distance [km]', fontsize = fs)
#メモリの文字サイズ設定
ax.tick_params(labelsize=fs)

#タイトル
if name_variable == "chla":
    ax.set_title('Distribution of Chla along Station line (preliminary)', fontsize = fs*1.3,  y = 1.12) 
elif name_variable == "temp":
    ax.set_title('Distribution of Temp along Station line (preliminary)', fontsize = fs*1.3,  y = 1.12) 
else:
    ax.set_title('Distribution of Salinity along ' + name_line +' line (preliminary)', fontsize = fs*1.3,  y = 1.12) 

fig.savefig("out.pdf", bbox_inches="tight", dpi=600)
#fig.savefig("out.eps", bbox_inches="tight", dpi=600)

